c 
c   > Do you havew some routines that can check if any arithmetic exceptions
c   > occured in a segment of code after that segment has been executed?
c   > I.e., do you have some routines that can test the arithemtic overflow
c   > and overflow flags and clear them?
c 
c Ken,
c   Sun actually provides these routines.  Try  
c       man f77_ieee_environment  
c       man ieee_flags
c for all of the information.   You can write programs like the following:
c 
c ( note:  this must be in a file called   pgm.F    the .F indicates
c that the preprocessor must process the file before the compiler is
c applied  (This is because of the #include statement.)
c 
c (You can do all of this in C too.)
c 
c 
c
c  The following include file defines the bit numbers for the 
c  possible floating point exceptions.
c
#include <f77/f77_floatingpoint.h>
c
c  Bit patterns for the different floating point exceptions.
c
      integer           noEX, divisionEX, inexactEX, invalidEX, 
     &                  overflowEX, underflowEX, seriousEX
      parameter       ( noEX = 0, 
     &                  divisionEX  = lshift( 1, fp_division ),
     &                  inexactEX   = lshift( 1, fp_inexact ),
     &                  invalidEX   = lshift( 1, fp_invalid ),
     &                  overflowEX  = lshift( 1, fp_overflow ),
     &                  underflowEX = lshift( 1, fp_underflow ),
     &                  seriousEX   = or( divisionEX,
     &                                or( underflowEX,
     &                                or( overflowEX,
     &                                    invalidEX ) ) ) )
c
c  Need to define progam variables before declaring the 
c  statement functions.
c
      real  x, y, z
      integer ieeer, ieee_flags
      character*16 out
c
c
c  Definitions of the statement functions used to manipulate 
c  floating point exception flags.
c
      logical fpException, underflow, overflow, invalid, zeroDivide
      integer EXflags
      fpException( EXflags ) = ( and( seriousEX, EXflags ) .ne. 0 )
      underflow( EXflags ) = ( and( underflowEX, EXflags ) .ne. 0 )
      overflow( EXflags ) = ( and( overflowEX, EXflags ) .ne. 0 )
      invalid( EXflags ) = ( and( invalidEX, EXflags ) .ne. 0 )
      zeroDivide( EXflags ) = ( and( divisionEX, EXflags ) .ne. 0 )
c
c  The program begins ...
c
c
c  Make sure that the floating point status flags are zero
c  before we start computing.
c
      ieeer = ieee_flags( 'clear', 'exception', 'all', out )
c
c  Do some computations that generate exceptions.
c 
      x = 1.0e+20
      y = 1.0e+21
      z = x * y
      print *, 'x = ', x, ' y = ', y, ' z = ', z
      x = 1.0e-20
      y = 1.0e-21
      z = x * y
      print *, 'x = ', x, ' y = ', y, ' z = ', z
      x = 1.0e+0
      y = 1.0e+1 / 1.0e+1
      z = 1.0e0 / ( x - y )
      print *, 'x = ', x, ' y = ', y, ' z = ', z
c
c  Check the exception flags.
c
      ieeer = ieee_flags( 'get', 'exception', 'all', out )
c
c  "Out" will contain the name of the exception of highest priority,
c  but ieeer will have a bit set for each exception that occured.
c
      print *, 'out = ', out, '  ieeer = ', ieeer
c
      if ( underflow( ieeer ) ) then
          print *, '...Underflow has occurred.'
      else
          print *, '...Underflow has not occurred.'
      end if 
      if ( overflow( ieeer ) ) then
          print *, '...Overflow has occurred.'
      else
          print *, '...Overflow has not occurred.'
      end if 
      if ( zeroDivide( ieeer ) ) then
          print *, '...zeroDivide has occurred.'
      else
          print *, '...zeroDivide has not occurred.'
      end if 
      if ( invalid( ieeer ) ) then
          print *, '...Invalid has occurred.'
      else
          print *, '...Invalid has not occurred.'
      end if 
c
c  Now clear the flags so we don't get a message about the
c  exceptions after our program finishes.
c
      ieeer = ieee_flags( 'clear', 'exception', 'all', out )
      end



