c Tanya Ruppell  009596962  CSC2307  Assignment#1


c Specification:  Assuming no arithmetic exceptions, the computed roots of
c the cubic P(x) will satisfy p(x) <= 2E where E is defined as follows:
c
c E=(machine eps)*(error term)-|computed function value|*1/2(machine eps)
c
c the derivation of the error term is included in the external
c documentation

c Method:  I used the bisection method to locate roots.  
c "Bound" locates an interval within which a root occurs. 
c
c "Bisection" finds a value within the interval which satifies the
c specification.
c
c "Deflation" uses composite deflation to deflate the cubic in the
c stabilest fashion.
c
c "quad" is Dr. Jackson`s subroutine
c
c "quad_err" takes the results from "quad", test them against the
c specification and set flags appropriately.
c 
c "polish" uses the Newton-Raphson method for polishing root estimates


c Sources:  The stopping criterion for the bisection is taken from
c Duane Adams' paper "A Stopping Criterion for Polynomial Root Finding.
c Since I was only looking for a real root it seemed logical to use as a
c stopping crierion the bound where computations are no longer reliable
c due to rounding errors.
c
c Peters and Wilkinson's "Practical Problems Arising in the Solution of
c Polynomial Equations" was the source for the deflation method.
c
c "Numerical Recipies in Fortran" was the source for the polishing code


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c  Routine which finds the roots of a cubic function, follows the outline
c  specified in the assignment

	SUBROUTINE Cubic(a, rr, ri, iflag)
	real a(0:3), rr(3), ri(3) 
	integer iflag(3)

	logical bounded, Bound
	real above, below
	real b(0:2)
	logical isNaN, isInf
	integer ind

c	  test coefficients for NaN and inf using Dr. Jackson's logical
c	  functions from quad1.f

	do 100 ind=0, 3, 1
	if (isNaN(a(ind)) .or. isInf(a(ind))) then
	   iflag(1)=-2
	   iflag(2)=-2
	   iflag(3)=-2
	   return
	endif
100	continue

c	  look for degenerate equations

	if (a(3) .eq. 0) then
	   if (a(2) .eq. 0) then
	      if (a(1) .eq. 0) then

c		  all coefficients are 0

		 if (a(0) .eq. 0) then
		    iflag(3)=0
		    iflag(2)=0
		    iflag(1)=0

c		  constant function (not 0)

		 else
		    iflag(3)=-1
		    iflag(2)=-1
		    iflag(1)=-1		    
		 endif

c		linear function

	      else
		 iflag(3)=-1
		 iflag(2)=-1
		 iflag(1)=1
		 rr(1)=-a(0)/a(1)
		 ri(1)=0
 	      endif

c	     quadratic function

	   else
	      call quad(a(2), a(1), a(0), rr(1), ri(1), rr(2), ri(2), fl)
	      call quad_err(a, rr, ri, iflag, fl)
	      iflag(3)=-1
	   endif

c	  cubic function

	else

c	    initialize flags

	   iflag(1)=-7
	   iflag(2)=-7
	   iflag(3)=-7

c	     locate an interval that contains a root

	   bounded=Bound(a, above, below, iflag(3))
 	   if (bounded) then
	      call bisect(a, above, below, iflag(3))

c		set the real root, rr(3), equal to one of the bounding
c		values (these values will be equal after bisection) and
c		deflate the cubic only if a root has been found or
c		approximated, otherwise set all flags to -6

	      if (iflag(3) .ne. -6) then
	      	 rr(3)=above
	      	 ri(3)=0
	      	 call deflate(a, b, above, iflag(2))
	      else
		 iflag(2)=-6
		 iflag(1)=-6
	      endif

c		find roots of resulting quadratic if there have not been
c		any arithmetic exceptions, otherwise set flag
c		appropriately

	      if (iflag(2) .ne. -6) then
	   	 call quad(b(2),b(1),b(0), rr(1),ri(1), rr(2),ri(2), fl)
		 call quad_err(a, rr, ri, iflag, fl)
	      else
	         iflag(1) = -6 
	      endif
	   else
	      iflag(1)=-6
	      iflag(2)=-6
	      iflag(3)=-6
	   endif
	endif

	return
	end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c   a routine to set flags based on the values returned from quad

	subroutine quad_err(arr, rroots, iroots, flags, fl_quad)
	real arr(0:3), rroots(3), iroots(3)
	integer flags(3), fl_quad

	logical Test_spec, spec	

c	  roots returned by quad are arbitrary

	if (fl_quad .eq. -2.) then
	   flags(1)=-6
	   flags(2)=-6

c	  there are infinately many roots

	else if (fl_quad .eq. -1.) then
	   flags(1)=0
	   flags(2)=0

c	  there are no roots

	else if (fl_quad .eq. 0.) then
	   flags(1)=-1
	   flags(2)=-1

c	  one root, normalized number.  It must be tested to determine if
c	  it meets the specification. If it does not, it is polished to
c	  improve the estimate

	else if (fl_quad .eq. 11.) then
	   spec=Test_spec(arr, rroots(1), iroots(1))
	   if (spec) then
	      flags(1)=1
	   else
	      call polish(arr, rroots(1), iroots(1), flags(1))
	   endif
	   flags(2)=-1

c	  one root, not normalized

	else if (fl_quad .eq. 10.) then
	   flags(1)=-3
	   flags(2)=-1

c	  two roots, both normalized numbers.  Both must be tested against
c	  the specification.  If the specification is not met, the roots
c	  are improved by polishing

	else if (fl_quad .eq. 22.) then
	   spec=Test_spec(arr, rroots(1), iroots(1))
	   if (spec) then
	      flags(1)=1
	   else
	      call polish(arr, rroots(1), iroots(1), flags(1))
	   endif
	   spec=Test_spec(arr, rroots(2), iroots(2))
	   if (spec) then
	      flags(2)=1
	   else
	      call polish(arr, rroots(2), iroots(2), flags(2))
	   endif

c	  one normalized root, one denormalized.  Test the normalized root
c	  against the specification.  The root is then polished, if it is
c	  not met.

	else if (fl_quad .eq. 21.) then
	   spec=Test_spec(arr, rroots(1), iroots(1))
	   if (spec) then
	      flags(1)=1
	   else
	      call polish(arr, rroots(1), iroots(1), flags(1))
	   endif
	   flags(2)=-3

c	  two denormalized roots

	else
	   flags(1)=-3
	   flags(2)=-3
	end if
	return
	end	   

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c   A routine to refine root estimates using Newton-Raphson Method

	subroutine polish(arr, r, i, flag)
	real arr(0:3), r, i
	integer flag  

	real p, p1, p_neg, p_pos
	complex x, c, c1
	logical Test_spec
	logical over, under, prob

c	  These values are used to ensure that we are actually moving
c	  closer to the root.  They will hold the minimum positive and
c	  negative values.

	p_neg=-10.**37
	p_pos=10.**37

	if (i .eq. 0.) then

c	     the value of the function at r is calculated

120	   p=arr(3)*r**3+arr(2)*r**2+arr(1)*r+arr(0)

c	    test for arithmetic exceptions
	
	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag=-5
	      return

c	    if an underfow occurs but p>0 then the part of the
c	    calculation that caused the underflow probably would have had
c	    little effect on the final result

	   elseif (under .and. p .eq. 0.) then
	      flag=-5
	      return
	   endif

c	    the value of the derivative at r is calculated

	   p1=3*arr(3)*r**2+2*arr(2)*r+arr(1)

c	    test for arithmetic exceptions

	   call exceptions(over, under, prob) 
	   if (over .or. prob) then
	      flag=-5
	      return

c	    if an underfow occurs but p1>0 then the part of the
c	    calculation that caused the underflow probably would have had
c	    little effect on the final result

	   elseif (under .and. p1 .eq. 0.) then
	      flag=-5
	      return
	   endif

c	    the derivative should not disappear

	   if (p1 .eq. 0.) then
	      flag=-5
	      return
	   endif


c	    test if we are getting farther away from the root by comparing
c	    to p_pos and p_neg then resetting the appropriate value for
c	    the next iteration 

	   if (p .gt. 0.) then
	      if (p .gt. p_pos) then
		 flag=-5
		 return
	      else
		 p_pos=p
	      endif
	   else
	      if (p .lt. p_neg) then
		 flag=-5
		 return
	      else
		 p_neg=p
	      endif
	   endif


c	    find the x-intercept of the linear approximation of the cubic
c	    at r

	   r=r-p/p1

c	    check for arithmetic exceptions

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag=-5
	      return
	   elseif (under .and. r .eq. 0.) then
	      flag=-5
	      return
	   endif


c	    loop until the specification is met

	   if (.not. Test_spec(arr, r, 0.)) go to 120


c	  perform the same procedure for complex roots

	else
	   x=cmplx(r, i)

c	    compute the value of the function

130	   c=arr(3)*x**3+arr(2)*x**2+arr(1)*x+arr(0)

c	    check for exceptions

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag=-5
	      return
	   elseif (under .and. c .eq. 0.) then
	      flag=-5
	      return
	   endif


c	    compute the value of the derivative

	   c1=3*arr(3)*x**2+2*arr(2)*x+arr(1)


c	    check for exceptions

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag=-5
	      return
	   elseif (under .and. c1 .eq. 0.) then
	      flag=-5
	      return
	   endif


c	    the derivative should not disappear

	   if (c1 .eq. 0.) then
	      flag=-5
	      r=real(x)
	      i=aimag(x)
	      return
	   endif

c	    check that we are moving toward a root

	   if (Mag(c) .gt. p_pos) then
	      flag=-5
	      return
	   else
	      p_pos=Mag(c)
	   endif


c	    compute th ex-intercept of the linear approximation at x

	   x=x-c/c1

c	    check for exceptions

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag=-5
	      return
	   elseif (under .and. x .eq. 0.) then
	      flag=-5
	      return
	   endif

c	    loop until the specification is met

	   if (.not. Test_spec(arr, real(x), aimag(x))) then
	      go to 130

c	    convert complex number back to reals

	   else
	      r=real(x)
	      i=aimag(x)
	   endif
	endif

	flag=1
	return
	end



cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c   A function to test if a root meets the program's specification

	logical function Test_spec(arr, r, i)
	real arr(0:3), r, i

	real ej, ek, bj, bk, s_cond, val, delta
	complex cbj, cbk, x
	real Mag
	integer count
	logical over, under, prob, err_b

c	  delta=1/2 machine epsilon

	delta=2.**(-24)

c	  used to keep track of underflows

	err_b=.false.


c	  evaluate function and error term iteratively for a real root

	if (i .eq. 0.) then

c	     evaluate function and error term iteratively, include
c	     checking for underflows

	   bj = arr(3)
	   ej = abs(bj)*delta
	   do 150 count=2, 0, -1
	     bk = r*bj + arr(count)
	     call exceptions(over, under, prob)
	     if (under)    err_b=.true.

	     ek=abs(r)*ej
	     ek=ek+(abs(r*bj)*delta+abs(arr(count)))*(2+delta)*delta

	     bj=bk
	     ej=ek
150	   continue

	   val=abs(bk)

c	    test if underflows have had an effect on calculations.  If
c	    there have been underflows and the function evaluates to 0,
c	    this is probably not the correct value.  However, if bk is
c	    not zero, then the underflow probably had very little effect.
c	    The error term should never be zero.

	   if (err_b .and. bk .eq. 0. .or. ek .eq. 0.) then
	      Test_spec=.false.
	      return
	   endif

c	    test for overflows

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      Test_spec=.false.
	      return
	   endif


c	  evaluate function and error term iteratively for a complex root
c	  (same procedure as for a real root)
	
	else
	   x=cmplx(r, i)
	   ej=Mag(x)
	   cbj=cmplx(arr(3), 0.)
	   do 160 count=2, 0, -1

c	       compute function and error term, keeping track of
c	       underflows

	     cbk = x*cbj + arr(count)
	     call exceptions(over, under, prob)
	     if (under)    err_b=.true.

             ek=Mag(x)*ej
	     ek=ek+(Mag(x*bj)*delta+Mag(arr(count)))*(2*delta)*delta

	     cbj=cbk
	     ej=ek
160	   continue
	   val = Mag(bj)

c	    check for disastrous underflows

	   if (err_b .and. cbk .eq. 0. .or. ek .eq. 0) then
	      Test_spec=.false.
	      return
	   endif

c	    check for overflows

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      Test_spec=.false.
	      return
	   endif

	endif

c	  stopping condition

	s_cond = 2*ej
	tp = abs(bk)


c	  test stopping condition (specification)

	if (s_cond .gt. val .or. s_cond .eq. val) then
	   Test_spec=.true.
	else
	   Test_spec=.false.
	endif
	
	return
	end


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c   function to return the magnitude of a complex number

	real function Mag(in)
	complex in

c	real t1, t2

c	t2=aimag(in)

c	print *, 'Mag', t1, t2

	Mag=sqrt(real(in)**2+aimag(in)**2)

	return
	end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c  Routine to deflate the cubic to a quadratic

	subroutine deflate(arr, brr, x, flag)
	real arr(0:3), brr(0:2), x
	integer flag

	real p(0:2), q(0:2), r, temp0, temp1, temp2
	logical over, under, prob


c	  determine "forward" division coefficients

	p(2)=arr(3)
	p(1)=arr(2)+p(2)*x
	p(0)=arr(1)+p(1)*x

	call exceptions(over, under, prob)
	if (over .or. prob) then
	   flag=-6
	   return

c	 check for disastrous underflows, those that make the coefficients
c	 zero.  There is no need to check p(2) because this is an
c	 assignment, not an arithmetic operation.

	elseif (under) then
	   if (p(1) .eq. 0.) then
	      temp0=p(2)*x
	      call exceptions(over, under, prob)
	      if (under) then
		 flag=-6
		 return
	      endif
	   endif

	   if (p(2) .eq. 0.) then
	      temp0=p(1)*x
	      call exceptions(over, under, prob)
	      if (under) then
		 flag=-6
		 return
	      endif
	   endif
	endif


c	  determine "backward" division coefficients

	q(0)=-arr(0)/x
	q(1)=(q(0)-arr(1))/x
	q(2)=(q(1)-arr(2))/x

	call exceptions(over, under, prob)
	if (over .or. prob) then
	   flag=-6
	   return


c	  check for disastrous underflows

	elseif (under) then 
	   if (q(0) .eq. 0.) then
	      temp0=-arr(0)/x
	      call exceptions(over, under, prob)
	      if (under) then
		 flag=-6
		 return
	      endif
	   endif

	   if (q(1) .eq. 0.) then
	      temp0=(q(0)-arr(1))/x
	      call exceptions(over, under, prob)
	      if (under) then
		 flag=-6
		 return
	      endif
	   endif

	   if (q(2) .eq. 0.) then
	      temp0=(q(1)-arr(2))/x
	      call exceptions(over, under, prob)
	      if (under) then
		 flag=-6
		 return
	      endif
	   endif
	endif


c	  evaluate functions which must be minimized to perform the
c	  stabilest possible deflation (mostly forward for minimum value
c	  root, mostly backward for maximum value root).  Test for
c	  division by zero before performing operations.

c	  Test for exceptions.  If an overflow occurs set the temp value
c	  to a maximum so that it doesn't affect the comparison to find a
c	  minimum value.  We do not need to check for underflows because
c	  this simply means we have a vry small value that would likely be
c	  the minimum if not for the underflow.  Handling of errors is
c	  straightforward in this section because these values are
c	  relative to each other and will not be used in any further
c	  computation.

	if (p(2) .ne. 0. .or. q(2) .ne. 0.) then
	   temp2=abs(p(2)-q(2))/(abs(p(2))+abs(q(2)))
	   call exceptions(over, under, prob)
	   if (over .or. prob)   temp2 = 10.**37
	else
	   temp2 = 10.**37
	endif
	if (p(1) .ne. 0. .or. q(1) .ne. 0.) then
	   temp1=abs(p(1)-q(1))/(abs(p(1))+abs(q(1)))
	   if (over .or. prob)   temp1 = 10.**37
	else
	   temp1 = 10.**37
	endif
	if (p(0) .ne. 0. .or. q(0) .ne. 0.) then
	   temp0=abs(p(0)-q(0))/(abs(p(0))+abs(q(0)))
	   if (over .or. prob)   temp0 = 10.**37
	else
	   temp0 = 10.**37
	endif


c	  determine minimum of above functions

	if (temp2 .lt. temp1) then
	   if (temp2 .lt. temp0) then
	      r=2.
	   else
	      r=0.
	   endif
	elseif (temp1 .lt. temp0) then
	   r=1.
	else
	   r=0.
	endif


c	  compose function out of the appropriate combination of forward
c	  and backward coefficients 

	brr(2)=p(2)

	if (r .eq. 0) then
	   brr(1)=p(1)
	   brr(0)=p(0)
	else if (r .eq. 1) then
	   brr(1)=p(1)/2+q(1)/2.

c	     check for exceptions.  In the case of an over flow, either
c	     coefficient will be an approximate, I choose to set brr(1) to
c	     p(1), it could also be set to q(1).  The rewriting of the
c	     equation should avoid underflows.

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      brr(1)=p(1)
	   elseif (under) then
	      brr(1)=(p(1)+q(1))/2
	   endif
	   brr(0)=q(0)
	else
	   brr(1)=q(1)
	   brr(0)=q(0)
	endif

	print *, q(2), q(1), q(0), p(2), p(1), p(0)
	print *, brr(2), brr(1), brr(0)

	return
	end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c  routine to bisect interval until a root is approximated

	subroutine bisect(arr, up, down, flag)
	real arr(0:3), up, down
	integer flag

	integer Sgn
	integer sgn_up, sgn_down, sgn_mid
	real tp, mid, prev_mid
	real bj, bk, ej, ek
	logical over, under, prob, err_b

c	  evaluate function at end points of the interval.  I do not check
c	  for exceptions because the function has already been evaluated
c	  at these points (in determining the interval) and not exceptions
c	  occured.

	tp = arr(0)+up*arr(1)+up**2*arr(2)+up**3*arr(3)
	sgn_up = Sgn(tp)
	tp = arr(0)+down*arr(1)+down**2*arr(2)+down**3*arr(3)
	sgn_down = Sgn (tp)

	prev_mid=up


c	  if loop is a single number, set flag to 1 (I don't include this
c	  as an else statement in the larger loop because it could result
c	  in the flag value being set twice.)


	if (up .eq. down)  flag=1

c	  loop until interval is reduced to a single number

200	if (up .ne. down) then
	   mid = up/2+down/2


c	    this avoids underflows by avoiding the small denormalized
c	    numbers (it is not necessary to do this but it is easier to
c	    check against flmin() than to check for an underflow)

	   if(abs(up)/2 .lt. flmin())  mid=(up+down)/2


c	     evaluate error term and function at midpoint (to be used in
c	     stopping condition

	   delta=2.**(-24)
	   err_b= .false.

	   bj = arr(3)
	   ej = abs(bj)*delta
	   do 210 i=2, 0, -1
	     bk = mid*bj + arr(i)
	     call exceptions(over, under, prob)
	     if (under)   err_b= .true.

             ek=abs(mid)*ej
	     ek=ek+(abs(mid*bj)*delta+abs(arr(i)))*(2+delta)*delta

c	       ignore underflows for now
	     call exceptions(over, under, prob)
	

	     bj=bk
	     ej=ek
210	   continue

c	    check for disastrous underflows in the calculation of bk.  The
c	    error term should never be zero, this alone signals that a
c	    disastrous underflow has occurred


	   if (err_b .and. bk .eq. 0. .or. ek .eq. 0.) then
	      flag=-5
	      return
	   endif


c	     test for arithmetic exceptions and set flag appropriately

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag = -6
	      return
	   endif


	   sgn_mid = Sgn(bk)


c	    stopping condition

	   s_cond = 2*ej
	   tp = abs(bk)

c	     test against stopping condition, if not met then move the
c	     appropriate end-point to the middle value and bisect again

	   if (tp .lt. s_cond .or. tp .eq. s_cond) then
	      up = mid
	      down = mid
	      flag = 1
	   elseif (sgn_mid .eq. sgn_up) then
	      up = mid
	      sgn_up = sgn_mid
	   else
	      down = mid
	      sgn_down = sgn_mid
	   endif
	   go to 200
	endif

	return
	end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c  Finds interval for a real root.

	logical function Bound (arr, up, down, flag)
	real arr(0:3), up, down
	integer flag

	real old_up, old_down, tp
c	real temp1, temp2, temp3
	integer sgn_up, sgn_down, old_sgn
	integer Sgn
	logical over, under, prob

c	  begin with the interval [-1, 1]

	up = 1
	down = -1
	old_up = -1
	old_down = 1


c	  evaluate function at endpoints, if one of these evaluates to 
c	  zero, we take that single point as the interval

c	  overflows arre tested for but not underflows.  An
c	  underflow signifies a value that is close to the root because
c	  only addition is used in these first calculations

	tp = arr(0)+arr(1)+arr(2)+arr(3)

	call exceptions(over, under, prob)
	if (over .or. prob) then
	   flag=-6
	   Bound=.false.
	   return
	endif

	sgn_up = Sgn(tp)

	if (sgn_up .eq. 0) then
	   down = up
	   Bound = .true.
	   flag=1
	   return
	endif

	tp = arr(0)-arr(1)+arr(2)-arr(3)

	call exceptions(over, under, prob)
	if (over .or. prob) then
	   flag=-6
	   Bound=.false.
	   return
	endif

	sgn_down = Sgn(tp)
	if (sgn_down .eq. 0) then
	   up = down
	   Bound = .true.
	   flag=1
	   return
	endif


c	  compare sign of end-points until function crosses zero

275	if (sgn_up .eq. sgn_down) then

c	     set values for comparison

	   old_up = up
	   old_down = down
	   old_sgn = sgn_up

c	     increase size of interval
	   up = 2*up
	   down = 2*down

	   tp = arr(0)+up*arr(1)+up**2*arr(2)+up**3*arr(3)

c	    check for arithmetic exceptions

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag=-6
	      Bound=.false.
	      return

c	    if an underflow occurs, simply keep the previous sign

	   elseif (under .and. tp .eq. 0.) then
	      sgn_up=sgn_up
	   else
	      sgn_up=Sgn(tp)
	   endif

	   tp = arr(0)+down*arr(1)+down**2*arr(2)+down**3*arr(3)

c	    check for arithmetic exceptions

	   call exceptions(over, under, prob)
	   if (over .or. prob) then
	      flag=-6
	      Bound=.false.
	      return

c	     in the case of an underflow, keep the same sign

	   elseif (under .and. tp .eq. 0.) then
	      sgn_up=sgn_up
	   else
	      sgn_down=Sgn(tp)
	   endif

	   go to 275
	endif


c	  test if an endpoint is zero

	if (sgn_up .eq. 0) then
	   down = up
	   Bound = .true.
	elseif (sgn_down .eq. 0) then
	   up = down
	   Bound = .true.

c	  decrease interval size by using only the part that introduced
c	  the zero crossing (most recent addition)

	elseif (sgn_down .eq. old_sgn) then 
	   down = old_up
	   Bound = .true.
	else
	   up = old_down
	   Bound = .true.
	endif
	
	end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c  a function to determine the sign of an argument

	integer function Sgn(arg)
	real arg

	if (arg .gt. 0.0) then
	   Sgn = 1
	elseif (arg .lt. 0.0) then
	   Sgn = -1
	else 
	   Sgn = 0
	endif
	return
	end
